<?php

define('RST_TIMES',        0x01);
define('RST_PERMS',        0x02);
define('RST_BYID',         0x04);
define('RST_BYNAME',       0x08);
define('RST_CHKTIME',      0x10);
define('RST_FULLREPORT',   0x20);
define('RST_ERR_REPORT',   0x40);
define('RST_SHORTREPORT',  0x80);
define('RST_TYPRESTORE',   0x100);
define('RST_TYPVERIFY',    0x200);
define('RST_EXIST',        0x400);
define('VRF_STD',          0x800);
define('RST_POSIXACLS',    0x1000);
define('RST_XATTRS',       0x2000);


require_once 'ArkeiaAction.php';

class RestorationController extends ArkeiaAction
{

    public function jsonStartAction($analyze=null)
    {
        $ctrlcfg = $this->_getCtrlCfg();
        $model = $this->_getModel($ctrlcfg);
        $idname = $ctrlcfg->id;

        $params = $this->_request->getPost();
        $listtrees = Zend_Json::decode($params['listtrees']);
        unset($params['listtrees']);

        /* unset selected value for redirection because when you 
           change value of combobox selected value is empty */
        unset($params['s_trcdest_selected']);
        unset($params['s_trcsrc_selected']);

        $params = $this->_clean_select_POST($params);
        $params = $this->_clean_POST($params);

        /* Set Restoration Options */
        $params['s_rstcr'] = RST_SHORTREPORT | RST_FULLREPORT | RST_TYPRESTORE;
        if (isset($params['RST_TIMES']) && $params['RST_TIMES'])
            $params['s_rstcr'] |= RST_TIMES;
        if (isset($params['RST_PERMS']) && $params['RST_PERMS'])
            $params['s_rstcr'] |= RST_PERMS;
        if (isset($params['RST_BY_NAME_OR_ID']) && 
                  $params['RST_BY_NAME_OR_ID'] == 'RST_BYNAME')
            $params['s_rstcr'] |= RST_BYNAME;
        else
            $params['s_rstcr'] |= RST_BYID;
        if (!isset($params['RST_CHKTIME']) || !$params['RST_CHKTIME'])
            $params['s_rstcr'] |= RST_CHKTIME;
        if (isset($params['RST_EXIST']) && $params['RST_EXIST'])
            $params['s_rstcr'] |= RST_EXIST;
        if (isset($params['RST_ACL']) && $params['RST_ACL'])
            $params['s_rstcr'] |= RST_POSIXACLS;
		if (isset($params['RST_XATTR']) && $params['RST_XATTR'])
            $params['s_rstcr'] |= RST_XATTRS;
        if (!isset($params['RST_TYPE']) || $params['RST_TYPE'] == 'RST_RST')
            $params['s_rstcr'] |= RST_TYPRESTORE;
        else {
            if ($params['RST_TYPE'] == 'RST_VRF_HIGH')
                $params['s_rstcr'] |= RST_TYPVERIFY;
            else
                if ($params['RST_TYPE'] == 'RST_VRF_STD')
                    $params['s_rstcr'] |= RST_TYPVERIFY | VRF_STD;
                else
                    $params['s_rstcr'] |= RST_TYPRESTORE;
        }
        unset($params['RST_TIMES']);
        unset($params['RST_PERMS']);
        unset($params['RST_BY_NAME_OR_ID']);
        unset($params['RST_CHKTIME']);
        unset($params['RST_EXIST']);
        unset($params['RST_ACL']);
        unset($params['RST_TYPE']);

	if(isset($params['optRestorationForm_forcemediaserveropt'])) {
		unset($params['optRestorationForm_forcemediaserveropt']);
		$params['MEDIA_SERVER'] = $params['optRestorationForm_forcemediaserversel'];
	}
	unset($params['optRestorationForm_forcemediaserversel']);

		for($i=0; $i<count($listtrees); $i++) {
			if(isset($listtrees[$i]['ISPOPUPED'])) {
				if($listtrees[$i]['ISPOPUPED'] == 'no' && isset($listtrees[$i]['BKSID'])) {
					unset($listtrees[$i]['BKSID']);
				}
				unset($listtrees[$i]['ISPOPUPED']);
			}
		}

        /* set tree */
        for ($i = 0; isset($listtrees[$i]); $i++) {
            if(isset($listtrees[$i]['FULLNAME']) && isset($listtrees[$i]['FULLPATH'])) {
                unset($listtrees[$i]['FULLPATH']);
                $listtrees[$i]['FULLNAME'] = $this->navpath_base64_decode($listtrees[$i]['FULLNAME']);
            }
            $params['item'.$i] = $listtrees[$i];
        }

	foreach ($params as $key => $value) {
	    if (preg_match('/^RstTapeLABEL_(.*)$/', $key, $match)) {
	        if (!isset($params['RstTapeSelect_'.$match[1]])) {
		    $params['item'.$i] = array('CID' => $params['RstTapeCID_'.$match[1]], 'TPID' => $params['RstTapeTPID_'.$match[1]], 'LABEL' => $params['RstTapeLABEL_'.$match[1]]);
		  $i++;
		}
		unset($params['RstTapeCID_'.$match[1]]);
		unset($params['RstTapeTPID_'.$match[1]]);
		unset($params['RstTapeLABEL_'.$match[1]]);
		unset($params['RstTapeSelect_'.$match[1]]);
	    }
	}

        require_once 'Zend/Json.php';
        //error_log('jsonStartAction, params: '.Zend_Json::encode($params));
        /* no file select */
        if (!$i) {
            $res = $this->_set_error_message($this->lang['rst_start_error_no_files_selected']);
        } else {
	  if (($params['s_rstcr'] & RST_TYPVERIFY) && !$analyze) {
		if(isset($params['MEDIA_SERVER'])) {
			unset($params['MEDIA_SERVER']);
		}
	        $data = $model->startVerify($params, ($params['s_rstcr'] & VRF_STD) ? 1 : 2,  $this->view->context);
            } else if ("analyze" == $analyze) {
		if(isset($params['MEDIA_SERVER'])) {
			unset($params['MEDIA_SERVER']);
		}
                $data = $model->analyze($params, $this->view->context);
            } else if ("listFiles" == $analyze || "listFiles_PREPRINT" == $analyze) {
		if(isset($params['MEDIA_SERVER'])) {
			unset($params['MEDIA_SERVER']);
		}
                $data = $model->listFiles($params, $this->view->context);
            } else {
                $data = $model->startRestore($params, $this->view->context);
            }
            //error_log('jsonStartAction, data: '.Zend_Json::encode($data));
            //error_log('jsonStartAction, context: '.Zend_Json::encode($this->view->context));
	
            $res = $this->_get_result($data, $this->view->context);
        }
	//error_log('jsonStartAction, res: '.Zend_Json::encode($res));
	if ("listFiles_PREPRINT" == $analyze) {

		$_SESSION['restorationhook'] = 'LISTFILE_PREPRINT';

        foreach ($res as $app => $app_data) {

            if(isset($res[$app]['PARAM2'])) {

                foreach ($res[$app]['PARAM2'] as $key1 => $param) {

                    if(isset($res[$app]['PARAM2'][$key1]->item)) {
                    
                        if(isset($res[$app]['PARAM2'][$key1]->item->NAME)) {

                            $res[$app]['PARAM2'][$key1]->item = array($res[$app]['PARAM2'][$key1]->item);

                        }

			$content_raw = $this->lang['rst_list_files_start'] . ' - ' . $this->lang['rst_list_files_end'] . ' - ' . $this->lang['rst_list_files_filename'] . "\n\n";

                        $content = '<table class="tableListFiles">';
                        $content .= '<tr>';
                        $content .= '<th>'.$this->lang['rst_list_files_start'].'</th>';
                        $content .= '<th>'.$this->lang['rst_list_files_end'].'</th>';
                        $content .= '<th>'.$this->lang['rst_list_files_filename'].'</td>';
                        $content .= '</tr>';

                        foreach($res[$app]['PARAM2'][$key1]->item as $msg) {

                            list($stape, $sseg, $etape, $eseg, $file) = preg_split('/\t/', $msg->NAME, 5);

			$content_raw .= $stape.' #'.$sseg . ' - ' . $etape.' #'.$eseg . ' - ' . $this->navpath_make_fullpath($file) . "\n\n";

                            $content .= '<tr>';
                            $content .= '<td>'.$stape.' #'.$sseg.'</td>';
                            $content .= '<td>'.$etape.' #'.$eseg.'</td>';
                            $content .= '<td>'.$this->navpath_make_fullpath($file).'</td>';
                            $content .= '</tr>';

                        }

                        $content .= '</table>';

                        $res[$app]['PARAM2'][$key1]->item = array(array('NAME' => $content));

                    }

                }

            }

        }

	}
        if(isset($_SESSION) && isset($_SESSION['restorationhook'])) {
            $_SESSION['restorationhookdata'] = $content_raw;
            $this->writerestorationhook();
        }
        $result = Zend_Json::encode($res);
	//error_log('jsonStartAction, result: '.$result);
        $this->getResponse()->appendBody($result);
        return;
    }

	private function writerestorationhook() {
		if(isset($_SESSION) && isset($_SESSION['restorationhook']) && isset($_SESSION['restorationhookdata'])) {
			$tmppath = dirname(__FILE__) . '/../../tmp';
			$path = $tmppath . '/restorationlogs';
			if(file_exists($tmppath)) {
				if(!file_exists($path)) {
					mkdir($path);
					chmod($path, 0777);
				}
				file_put_contents($path . '/' . $_SESSION['restorationhook'], $_SESSION['restorationhookdata']);
				chmod($path . '/' . $_SESSION['restorationhook'], 0666);
			}
		}
	}

	public function jsonListFileDownLoadAction() {
		if(isset($_SESSION) && isset($_SESSION['restorationhook']) && isset($_SESSION['restorationhookdata'])) {
			$tmppath = dirname(__FILE__) . '/../../tmp';
			$path = $tmppath . '/restorationlogs';
			$file = $path . '/' . $_SESSION['restorationhook'];
			$tmp = explode('_', $_SESSION['restorationhook']);
			$filename = $tmp[0];
			if(file_exists($tmppath) && file_exists($path) && file_exists($file)) {
				$taille=filesize("$file");
				header("Content-Type: application/force-download; name=\"$filename.txt\"");
				header("Content-Transfer-Encoding: binary");
				header("Content-Length: $taille");
				header("Content-Disposition: attachment; filename=\"$filename.txt\"");
				header("Expires: 0");
				header("Cache-Control: no-cache, must-revalidate");
				header("Pragma: no-cache");
				readfile("$file");
				exit();
			}
		}
	}

    public function jsonAnalyzeAction()
    {
        $this->jsonStartAction("analyze");
        return;
    }
    
    public function jsonListFilesAction()
    {
        $this->jsonStartAction("listFiles");
        return;
    }

    public function jsonListFilesPrePrintAction()
    {
        $this->jsonStartAction("listFiles_PREPRINT");
        return;
    }



    public function showRestorationAction()
    {
        $this->displayFromScreenIni('showrestoration');
    }

    public function jsonShowRestorationAction()
    {
        $params = $this->_request->getPost();
        require_once 'Zend/Json.php';

        //error_log("jsonShowRestorationAction POST: ".Zend_Json::encode($this->_request->getPost()));
	
        if (!$params['PARAM2']) {
            $array_context = $this->view->context;
            list($id, $context) = $this->_getIdContext($array_context, 'ID_PORT');
            if ($id) {
                list($params['PARAM2'], $params['PARAM3']) = split("_", $id, 2);
            }
        }

        $this->view->showBackup = $params;

        $data = $this->displayFromScreenIni('showrestoration', FALSE);
        $this->_render_json_view($data);
        return;
    }

    public function jsonGetNextListFilesPrePrintAction() {
        require_once 'Zend/Json.php';
        $context = $this->view->context;
        $result = $this->_getNext($context);

        if ($result == null) return null;
        if (is_array($result)) {
            foreach ($result as $key => $val) {
                if ($key == "local") continue;
                if ($context) {
                    $result[$key]->param3 = $result[$key]->param4;
                    unset($result[$key]->param4);
                } else {
                    $result[$key]->param2 = $result[$key]->param3;
                    unset($result[$key]->param3);
                }
            }
        } else {
            if ($context) {
                $result->param3 = $result->param4;
            } else {
                $result->param2 = $result->param3;
            }
        }

        $res = $this->_get_result($result, $this->view->context);

        foreach ($res as $app => $app_data) {

            if (isset($res[$app]['PARAM2'])) {

                $PARAM2 = $res[$app]['PARAM2'];

                if ($PARAM2->item->NAME) {
                    $PARAM2->item = array($PARAM2->item);
                }

                foreach ($PARAM2 as $key1 => $param) {

                    if (isset($PARAM2[$key1]->item)) {

                        $content = '<table class="tableListFiles">';
                        $content .= '<tr><th>'.$this->lang['rst_list_files_start'];
                        $content .= '</th><th>'.$this->lang['rst_list_files_end'];
                        $content .= '</th><th>'.$this->lang['rst_list_files_filename'];
                        $content .= '</td></tr>';
    
                        foreach ($PARAM2[$key1]->item as $key => $msg) {

                            list($stape, $sseg, $etape, $eseg, $file) = preg_split('/\t/', $msg->NAME, 5);

                            $content .= '<tr><td>'.$stape.' #'.$sseg.'</td><td>'.$etape.' #'.$eseg.'</td><td>'.$this->navpath_make_fullpath($file).'</td></tr>';

                        }

                        $content .= '</table>';

                        $PARAM2[$key1]->item = array(array('NAME' => $content));

                    }

                }

            }

	    }

        $result = Zend_Json::encode($res);
        $this->getResponse()->appendBody($result);
        return;
    }

}

?>
