<?php

require_once 'ArkeiaAction.php';
require_once 'Arkeia/Soap.php';

class SoapController extends ArkeiaAction
{
    public function init()
    {
        $this->view = $this->getInvokeArg('view');
        $this->mvc = $this->getInvokeArg('mvc');
        $this->viewdir = $this->getInvokeArg('viewdir');
        $this->view->title = "Soap";
        $cfg = new Zend_Config_Ini('../app/config/soap.ini', 'default');
        $this->soapdebug = $cfg->soapdebug;
    }

    public function new_trans($trans, $trans_version, $transnum, $vector, $context=null)
    {
        $acx = Arkeia_Soap::getInstance();
        if ($this->view->auth->hasIdentity() && 
            ($id=$this->view->auth->getIdentity())) {
            $context=$id['appliances'];
        } else {
            $context = null;
        }
        if ($context) {
            $mylist = array();
            if (is_array($context->item)) $items = $context->item;
            else $items = array($context->item);
            foreach ($items as $item) {
                $sub = array();
                foreach ($item as $key => $val) {
                    $sub[$key] = $val;
                }
                array_push($mylist, $sub);
            }
            $soap_context = $acx->list2soap($mylist);
        }
        else $soap_context = $context;
        $result = $acx->new_trans($trans, $trans_version, $transnum, $vector, $soap_context);
        return $result;
    }

    private function _assign_result($acx, $result) 
    {
        if ($result) $this->view->assign('result',$result);
        if ($acx) {
            $request = $acx->getLastRequest();
            $response = $acx->getLastResponse();
            if ($request) $this->view->assign('request',$request);
            if ($response) $this->view->assign('response',$response);
        }
    }

    public function indexAction()
    {
        $this->view->assign('subaction','soap/summary');
        echo $this->view->render('layout.phtml');
    }

    public function testAction()
    {
        $this->view->assign('subaction','index/test');
        echo $this->view->render('layout.phtml');
    }

    public function summaryAction()
    {
        $this->view->assign('subtitle',"Soap Testing");
        echo $this->render();
    }

    public function dologin($sessnum)
    {
        $this->view->assign('subtitle',"Login");
        $acx = Arkeia_Soap::getInstance();
        $result = $acx->login($this->soapdebug->user,
                $this->soapdebug->passwd, $this->soapdebug->admin, $sessnum);
        $this->_assign_result($acx, $result);
        echo $this->render('summary');
    }

    public function loginAction()
    {
        $this->dologin(1);
    }

    public function login2Action()
    {
        $this->dologin(2);
    }

    public function logoutAction()
    {
        $this->view->assign('subtitle',"Logout");
        $acx = Arkeia_Soap::getInstance();
        $result = $acx->logout();
        $this->_assign_result($acx, $result);
        echo $this->render('summary');
    }

    public function jobsAction() {
        $this->view->assign('subtitle',"Jobs");
        $acx = Arkeia_Soap::getInstance();
        $trans = "BKP_LIST_ACTIVE";
        $trans_version = 2;
        $orig_list = array( 
            "NAME" => "mylist",
            "ID" => "123456",
            array (
                "Toto01" => "item01",
                "Test" => "myprop01",
            ),
            array (
                "NAME02" => "item02",
                "PROP02" => "myprop02",
            )
        );
        $mylist = array();
        foreach ($orig_list as $key => $val) {
            if (is_array($val)) {
                array_push($mylist, new SoapVar($val, 
                    SOAP_ENC_OBJECT, "xsd:list", null, "item")
                );
            } else {
                $mylist[$key] =  $val;
            }
        }
        $soaplist = new SoapVar($mylist, SOAP_ENC_OBJECT, "xsd:list");
        $vector = array( "param0" => $soaplist);
        $result =  $acx->new_trans($trans, $trans_version, 1,$vector);
        $this->_assign_result($acx, $result);
        echo $this->render('summary');
    }

    public function tapes($sessnum)
    {
        $this->view->assign('subtitle',"Tapes");
        $this->viewdir = $this->getInvokeArg('viewdir');
        $acx = Arkeia_Soap::getInstance();
        $trans = "TP_LIST_ALL";
        $trans_version = 2;
        $snd_all = 1;
        $vector = array("param0" => $snd_all);
        $result    = $this->new_trans($trans, $trans_version, $sessnum, $vector);
        $this->_assign_result($acx, $result);
        echo $this->render('summary');
    }

    public function tapesAction() {
        $this->tapes(1);
    }

    public function tapes2Action() {
        $this->tapes(2);
    }

    public function tapetypes($sessnum) {
        $this->view->assign('subtitle',"Tape Types");
        $this->viewdir = $this->getInvokeArg('viewdir');
        $acx = Arkeia_Soap::getInstance();
        $trans = "TP_LIST_TYPE";
        $trans_version = 2;
        $snd_all = 1;
        $vector = array("param0" => $snd_all);
        $result    = $this->new_trans($trans, $trans_version, $sessnum, $vector);
        $this->_assign_result($acx, $result);
        echo $this->render('summary');
    }
    public function tapetypesAction() {
        $this->tapetypes(1);
    }
    public function tapetypes2Action() {
        $this->tapetypes(2);
    }



    public function validateLogin($sessnum) {
        if ($this->soapdebug->admin) {
            $trans = "LDAP_LOGIN";
        } else {
            $trans = "USR_VALIDATE";
        }
        $vector = array("admin"=>$this->soapdebug->admin) ;
        $trans_version = 1;
        $this->view->subtitle = $trans;
        $acx = Arkeia_Soap::getInstance();
        $result = $acx->new_trans($trans, $trans_version, $sessnum, $vector);
        $this->_assign_result($acx, $result);
        echo $this->render('summary');
    }

    public function validateLoginAction() {
        $this->validateLogin(1);
    }

    public function validateLogin2Action() {
        $this->validateLogin(2);
    }

    public function getNext($sessnum) {
        $this->view->subtitle = "getNext";
        $acx = Arkeia_Soap::getInstance();
        $result = $acx->getNext(1,$sessnum);
        $this->_assign_result($acx, $result);
        echo $this->render('summary');
    }

    public function getNextAction() {
        $this->getNext(1);
    }

    public function getNext2Action() {
        $this->getNext(2);
    }

    public function drivesAction() {
        $this->view->assign('subtitle',"Drives");
        $acx = Arkeia_Soap::getInstance();
        Zend_Loader::loadClass(ArkDrive);
        $model = new ArkDrive();
        $result = $model->getAll();
        $this->_assign_result($acx, $result);
        echo $this->render('summary');
    }

    public function savepacksAction() {
        $this->view->assign('subtitle',"Savepacks");
        $acx = Arkeia_Soap::getInstance();
        Zend_Loader::loadClass(ArkSavepack);
        $model = new ArkSavepack();
        $result = $model->_getAll();
        $this->_assign_result($acx, $result);
        echo $this->render('summary');
    }

    public function tapeInfoAction() {
        $this->view->assign('subtitle', "Tape Information");
        $acx = Arkeia_Soap::getInstance();
        Zend_Loader::loadClass(ArkTape);
        $model = new ArkTape($this->mvc);
        $context = array(array(
            "APPLIANCE_GUID" => "2",
            "APPLIANCE_NAME" => "alexandre_1",
            "APPLIANCE_IP" => "10.1.11.11")
        );
        $result = $model->getResultOne("45dc68f1", $context);
        $this->_assign_result($acx, $result);
        echo $this->render('summary');
    }

    public function bkpListAllAction() {
        $this->view->assign('subtitle',"All Backup Done");
        $acx = Arkeia_Soap::getInstance();
        Zend_Loader::loadClass(ArkBackup);
        $model = new ArkBackup();
        $result = $model->listAll();
        $this->_assign_result($acx, $result);
        echo $this->render('summary');
    }

    public function bkpListSomeAction() {
        $this->view->assign('subtitle',"10 last Backup Done");
        $acx = Arkeia_Soap::getInstance();
        Zend_Loader::loadClass(ArkBackup);
        $model = new ArkBackup();
        $result = $model->listSome(10);
        $this->_assign_result($acx, $result);
        echo $this->render('summary');
    }
    public function bkpStartAction() {
        $this->view->assign('subtitle',"Start Backup");
        $acx = Arkeia_Soap::getInstance();
        Zend_Loader::loadClass(ArkBackup);
        $model = new ArkBackup();
        $params = array( 
            "SKID" => "453f36de",
            "DKID" => "4540a5af",
            "PLID" => "4540a5b8",
            "TYPE" => "TOTAL",
            "RET_TIME" => "30",
            "RET_TIME_UNIT" => "D"
        );
        $result = $model->startBackup($params);
        $this->_assign_result($acx, $result);
        echo $this->render('summary');
    }
    public function ldapSearchAction() {
        $this->view->assign('subtitle',"LDAP Search");
        $acx = Arkeia_Soap::getInstance();
        Zend_Loader::loadClass(ArkLdap);
        $model = new ArkLdap();
        $result = $model->search(null);
        $this->_assign_result($acx, $result);
        echo $this->render('summary');
    }

    public function ldapGetAttributesAction() {
        $this->view->assign('subtitle', "LDAP Get Attributes");
        $acx = Arkeia_Soap::getInstance();
        $trans = "LDAP_GET_ATTRIBUTES";
        $trans_version = 2;
        $vector = array( "param0" => "ou=USA,dc=arkeia,dc=com");
        $result =  $acx->new_trans($trans, $trans_version, 1,$vector);
        $this->_assign_result($acx, $result);
        echo $this->render('summary');
    }

    public function ldapAddApplianceAction() {
        $this->view->assign('subtitle', "LDAP Add Appliance");
        $acx = Arkeia_Soap::getInstance();
        $trans = "LDAP_ADD_APPLIANCE";
        $trans_version = 2;
        $orig_list = array(
                "cn" => "SoapAppliance01",
                "applianceIp" => "10.1.11.11",
        "manager" => array ("cn=Gabriel,ou=dev,ou=France,dc=arkeia,dc=com", "cn=Philippe,ou=sales,ou=France,dc=arkeia,dc=com")
        );
    $mylist = array();
        foreach ($orig_list as $key => $val) {
            if (is_array($val)) {
        $i = 0;
                foreach ($val as $a_key => $a_val) {
            $mylist[$key][$i++] = $a_val;
        }
            } else {
                $mylist[$key] =  $val;
            }
        }
        $soaplist = new SoapVar($mylist, SOAP_ENC_OBJECT, "xsd:list");
        $vector = array( "param0" => "ou=USA,dc=arkeia,dc=com", "param1" => $soaplist);
        $result =  $acx->new_trans($trans, $trans_version, 1,$vector);
        $this->_assign_result($acx, $result);
        echo $this->render('summary');
    }

}

?>
