dojo.provide('arkeia.console');

dojo.require('firebug/firebug');

/**
 * Detect if Firebug is present to profit of all its functionalities in debug mode
 * 
 * 
 * Arkeia console add an ignore list initialized from configuration and updated with 
 * 
 * 
 * DOJO 1.0 include an embedded cross-browser Firebug lite 
 * enabling somes basic Firebug features
 * 
 * The use of arkeia.info, arkeia.warn, and arkeia.error functions is so highly 
 * recommended in place of dojo.debug wich will disapear
 * 
 * You should also send several parameters for each var you want to evaluate
 * instead of sending a unique concatenated String
 * Json transformation is then no more needed
 * 
 * More infos about Firebug's console : http://www.getfirebug.com/console.html
 * 
 * @TODO implement missing features
 */
arkeia.console = {
    _name: '[arkeia.console]',
    showLog: true,
    showDebug: true,
    showInfo: true,
    showWarning: true,
    showError: true,
    groupNames: [],
    ignoreList: ('ignoreList' in arkeia.configuration) ? arkeia.configuration.ignoreList : {},
    ignore: function () {},
    show: function () {},
    log: function(){},
    debug: function(){},
    info: function(){},
    warning: function(){},
    error: function(){},
    group: function(){},
    groupEnd: function(){},
    action: function() {},
    setDebugMode: function(){},
    toggleConsole: function(){}
}


/**
 * arkeia.console.ignoreGroup
 * 
 * Ignore all messages (log, info, debug, warn) from a group
 * error messages can't be ignored
 * 
 * @param {String} groupName
 */
arkeia.console.ignoreGroup = function (groupName) {
    arkeia.console.ignore[groupName] = true;
};

/**
 * arkeia.console.showGroup
 * 
 * Stop ignoring messages 
 * 
 * @param {String} groupName
 */
arkeia.console.showGroup = function (groupName) {
    if (groupName in arkeia.console.ignore) {
        delete arkeia.console.ignore[groupName];
    }
};

/**
 * arkeia.log
 * 
 * Writes a message to the console. You may pass as many arguments as you'd like, 
 * and they will be joined together in a space-delimited line.
 * 
 * @param {Multiple} Message and variables to evaluate
 */
arkeia.log = function(){
    arkeia.console.action('log', arguments);
};

/**
 * arkeia.debug
 * 
 * Writes a message to the console. You may pass as many arguments as you'd like, 
 * and they will be joined together in a space-delimited line.
 * The message should also include a hyperlink to the line where it was called.
 * 
 * @param {Multiple} Message and variables to evaluate
 */
arkeia.debug = function(){
    arkeia.console.action('debug', debug.arguments);
};
dojo.debug = function (message) {
    if (arkeia.debugMode === 'off') {
        return false;
    }
    // arkeia.warn('"dojo.debug" is depreciated ! Please use "arkeia.debug", ".log", ".warn", or ".error" instead.');
    if (message.substr(0, 5) === 'FATAL' || message.substr(0, 6) === 'failed') {
        arkeia.error(message);
    } else if (message.substr(0, 6) === 'DEBUG: ') {
        arkeia.debug(message.substr(7));
    } else {
        arkeia.log(message);
    }
};

/**
 * arkeia.info
 * 
 * Writes a message to the console. You may pass as many arguments as you'd like, 
 * and they will be joined together in a space-delimited line
 * with the potentials visual "info" icon and color coding.
 * The message should also include a hyperlink to the line where it was called.
 * 
 * @param {Multiple} Message and variables to evaluate
 */
arkeia.info = function(){
    arkeia.console.action('info', arkeia.info.arguments);
};

/**
 * arkeia.warn
 * 
 * Writes a message to the console. You may pass as many arguments as you'd like, 
 * and they will be joined together in a space-delimited line
 * with the potentials visual "warning" icon and color coding.
 * The message should also include a hyperlink to the line where it was called.
 * 
 * @param {Multiple} Message and variables to evaluate
 */
arkeia.warn = function(){
    arkeia.console.action('warning', arkeia.warn.arguments);
};

/**
 * arkeia.error
 * 
 * Writes a message to the console. You may pass as many arguments as you'd like, 
 * and they will be joined together in a space-delimited line
 * with the potentials visual "error" icon and color coding.
 * The message should also include a hyperlink to the line where it was called.
 * 
 * @param {Multiple} Message and variables to evaluate
 */
arkeia.error = function(){
    arkeia.console.action('error', arkeia.error.arguments);
};

/**
 * arkeia.group
 * 
 * All next messages will be sent in a sub group
 * 
 * @param {Multiple} Message and variables to evaluate
 */
arkeia.group = function(groupName){
    arkeia.console.groupNames.unshift(groupName);
    arkeia.console.group(groupName);
};

/**
 * arkeia.groupEnd
 * 
 */
arkeia.groupEnd = function(){
    arkeia.console.groupNames.shift();
    arkeia.console.groupEnd();
};


/**
 * arkeia.console.action
 * 
 * Common comportement used by :
 * arkeia.log, arkeia.debug, arkeia.info, arkeia.warn, and arkeia.error 
 * 
 * @param {String} groupName
 */
arkeia.console.action = function(actionName, args) {
    var i = 0;
    var strArgs = '';
    if (arkeia.console['show' + actionName[0].toUpperCase() + actionName.substr(1)]) {
        for (i; i < arkeia.console.groupNames.length; i += 1) {
            if (arkeia.console.groupNames[i] in arkeia.console.ignoreList) {
                return false;
            }
        }
        for (i = 0; i < args.length; i += 1) {
            strArgs += ', args[' + i + ']';
        }
        eval('arkeia.console.' + actionName + '(' + strArgs.substr(2) + ');');
        return true;
    }
    return false;
};


arkeia.console.toggle = function (bool) {
    
}


/**
 * arkeia.console.action
 * 
 * Common comportement used by :
 * arkeia.log, arkeia.debug, arkeia.info, arkeia.warn, and arkeia.error 
 * 
 * @param {String} groupName
 */
arkeia.console.setDebugMode = function (debugMode) {
    arkeia.debugMode = debugMode;
    if (debugMode === 'off') {
        // Ignore debug messages
        arkeia.console.log = function(){};
        arkeia.console.debug = function(){};
        arkeia.console.info = function(){};
        arkeia.console.warning = function(){};
        arkeia.console.error = function(){};
        arkeia.console.group = function(){};
        arkeia.console.groupEnd = function(){};
        if ('console' in window && 'info' in window.console) {
            window.console.info('mode debug off');
        }
    } else if ('console' in window && 'debug' in window.console) {
        arkeia.console.log = window.console.log;
        arkeia.console.debug = window.console.debug;
        arkeia.console.info = window.console.info;
        arkeia.console.warning = window.console.warn;
        arkeia.console.error = window.console.error;
        arkeia.console.group = window.console.group;
        arkeia.console.groupEnd = window.console.groupEnd;
        if ('getFirebugURL' in window.console) {
            window.console.getFirebugURL = function() {
                return (arkeia.jsPath + 'firebug/');
            };
        }
        arkeia.info('mode debug on');
    }
    arkeia.console.toggle((debugMode === 'on'));
};

arkeia.console.setDebugMode(arkeia.debugMode);