dojo.provide('arkeia.popup');

dojo.require('arkeia.init');
dojo.require('arkeia.lang');
dojo.require('dojo.widget.FloatingPane');
dojo.require('arkeia.widget.ArkeiaButton');


arkeia.popup = function (msg, params) {
    if (!params) {
        params = {};
    }
    return arkeia.dialog(msg, params);
};

arkeia.alert = function (msg, callback, params) {
    if (!params) {
        params = {};
    }
    params.ok = true;
    if (!params.title) {
        params.title = arkeia.lang.getMsg('popup_label_alert');
    }
    if (callback) {
        params.callback = callback;
    }
    return arkeia.dialog(msg, params);
};

arkeia.confirm = function (msg, callback, params) {
    if (!params) {
        params = {};
    }
    params.confirm = true;
    params.cancel = true;
    if (!params.title) {
        params.title = arkeia.lang.getMsg('popup_label_confirm');
    }
    if (callback) {
        params.callback = callback;
    }
    return arkeia.dialog(msg, params);
};

arkeia.prompt = function (msg, input, callback, params) {
    params.input = input;
    params.submit = true;
    params.cancel = true;
    if (!params.title) {
        params.title = arkeia.lang.getMsg('popup_label_prompt');
    }
    if (callback) {
        params.callback = callback;
    }
    return arkeia.dialog(msg, params);
};


arkeia.dialog = function (msg, paramsList) {
    var _name = '[arkeia.dialog]';
    var dialog = {};
    var dialogChilds = {
        msg: msg
    };
    var dialogButtons = {};
    var dialogButtonParams = {};
    var paramName = '';
    var dialogStyle = {
        width: '300px',
        height: '200px',
        overflow: 'hidden'
        //,display: 'none'
    };
    var dialogParams = {
        bgColor: 'white',
        bgOpacity: '0.5',
        toggle: 'fade',
        toggleDuration: '250',
        displayCloseAction: true,
        title: arkeia.lang.getMsg('form_update_dialog_titlebar'),
        hasShadow: true,
        executeScripts: true,
        titleHeight: 20,
        resizable: false
    };
    var dialogDiv = dojo.doc().createElement('div');
    var content = dojo.doc().createElement('div');
    var pre = {};
    var tmpNode = {};

    arkeia.group(_name, '(msg', msg, 'paramsList:', paramsList, ')');
    dialogDiv.style.display = 'none';
    for (paramName in paramsList) {
        switch (paramName) {
        case 'title' :
            dialogParams.title = paramsList[paramName];
            break;
        case 'ok' : // break intentionnaly ommited
        case 'submit' : // break intentionnaly ommited
        case 'confirm' : // break intentionnaly ommited
        case 'cancel' : // break intentionnaly ommited
            tmpNode = dojo.doc().createElement('div');
            dialogButtonParams.caption = arkeia.lang.getMsg('popup_button_' + paramName);
            dialogButtonParams.onClick = function() {
                this.parent.destroy();
                if (paramName !== 'cancel' && 'callback' in paramsList) {
                    paramsList.callback(paramName);
                }
            };
            dialogButtons[paramName] = dojo.widget.createWidget('arkeia:ArkeiaButton', dialogButtonParams, tmpNode);
            insideWidgets.push(dialogButtons[paramName]);
            break;
        default:
            arkeia.warn('This param is unknown : ', paramName);
            break;
        }
    }
    if ('html' in paramsList && paramsList.html === false) {
        pre = dojo.doc().createElement('pre')
        pre.appendChild(dojo.doc().createTextNode(dialogChilds.msg));
        content.appendChild(pre);
    } else {
        content.innerHTML = dialogChilds.msg;
    }
    // Add widget input
    if ('input' in paramsList) {
        content.appendChild(paramsList.input.domNode);
    }
    // Add widget buttons
    if ('cancel' in paramsList) {
        content.appendChild(dialogButtons.cancel.domNode);
    }
    if ('confirm' in paramsList) {
        dialog.domNode.appendChild(dialogButtons.confirm.domNode);
    }
    if ('submit' in paramsList) {
        content.appendChild(dialogButtons.submit.domNode);
    }
    if ('ok' in paramsList) {
        content.appendChild(dialogButtons.ok.domNode);
    }
    if (!'insideWidgets' in paramsList) {
        paramsList.insideWidgets = [];
    }
    arkeia.info('dojo.body() : ', dojo.body());
    dojo.body().appendChild(dialogDiv);
    dialog = dojo.widget.createWidget('ModalFloatingPane', dialogParams, dialogDiv);
    dialog.containerNode.appendChild(content);
    dialog.onDestroy = function() {
        var widgets = insideWidgets;
        var toDestroy = {};
        while (insideWidgets.length) {
            toDestroy = insideWidgets.shift();
            if (toDestroy.widgetId) {
                toDestroy.destroy();
            }
        }
    };
    content.style.display = '';
    arkeia.groupEnd();
    return dialog;
}