dojo.provide("arkeia.widget.ArkeiaTimePicker");
dojo.require("dojo.widget.*");
dojo.require("dojo.widget.HtmlWidget");
dojo.require("dojo.event.*");
dojo.require("dojo.date.serialize");
dojo.require("dojo.date.format");
dojo.require("dojo.dom");
dojo.require("dojo.html.style");
dojo.require("arkeia.widget.ArkeiaIntegerSpinner");

dojo.requireLocalization("dojo.i18n.cldr", "gregorian");
dojo.requireLocalization("dojo.widget", "TimePicker");

dojo.widget.defineWidget(
	"arkeia.widget.ArkeiaTimePicker",
	dojo.widget.HtmlWidget,
	function(){

		this.time = "";

		this.storedTime = "";

		this.currentTime = {};
		
		this.classNames = {

			selectedTime: "selectedItem"
		};

		this.selectedTime = {
			hour: "",
			minute: ""
		};

	},
{
	isContainer: false,
	templatePath: dojo.uri.moduleUri("arkeia.widget", "templates/ArkeiaTimePicker.html"),
	templateCssPath: dojo.uri.moduleUri("arkeia.widget", "templates/ArkeiaTimePicker.css"),

	postMixInProperties: function(localProperties, frag) {

		arkeia.widget.ArkeiaTimePicker.superclass.postMixInProperties.apply(this, arguments);

	},

	fillInTemplate: function(args, frag){

		var source = this.getFragNodeRef(frag);
		dojo.html.copyStyle(this.domNode, source);

		if(args.value){
			if(args.value instanceof Date){
				this.storedTime = dojo.date.toRfc3339(args.value);
			}else{
				this.storedTime = args.value;
			}
		}

        hsnArgs = { signed: 'never', min: 0, max: 23, value: 0, size: 2, disabled: true, disabledarrows: false, templateCssPath: dojo.uri.moduleUri("arkeia.widget", "templates/ArkeiaTimePickerSpinner.css")};
        this.hourSpinnerNode = dojo.widget.createWidget("arkeia:ArkeiaIntegerSpinner", hsnArgs, this.hourContainerNode, "child");

        msnArgs = { signed: 'never', min: 0, max: 59, value: 0, size: 2, disabled: true, disabledarrows: false, templateCssPath: dojo.uri.moduleUri("arkeia.widget", "templates/ArkeiaTimePickerSpinner.css")};
        this.minuteSpinnerNode = dojo.widget.createWidget("arkeia:ArkeiaIntegerSpinner", msnArgs, this.minuteContainerNode, "child");

        bargs = {caption: "OK", isContainer: false, templateCssPath: dojo.uri.moduleUri("arkeia.widget", "templates/ArkeiaTimePickerButton.css")};
        this.buttonNode = dojo.widget.createWidget("Button", bargs, this.buttonContainerNode, "child");

		this.initData();
		this.initUI();
	},

	initData: function() {
        this.time = arkeia.widget.ArkeiaTimePicker.util.fromRfcDateTime(this.storedTime);
	},

	initUI: function() {

        var hour = this.time.getHours();
        var minute = this.time.getMinutes();

        this.hourSpinnerNode.setValue(hour);
        this.minuteSpinnerNode.setValue(minute);

        dojo.event.connect(this.buttonNode, "onClick", this, "onSetTime");

        this.onSetTime();

	},

	setTime: function(date) {

		if(date) {
			this.setDateTime(dojo.date.toRfc3339(date));
		}

		this.initData();
		this.initUI();
	},

	setDateTime: function(rfcDate) {
		this.storedTime = rfcDate;
	},

	onClick: function(evt) {
		dojo.event.browser.stopEvent(evt);
	},

	onSetTime: function() {

        var hour = this.hourSpinnerNode.getValue();
        var minute = this.minuteSpinnerNode.getValue();

        this.time = new Date();
        this.time.setHours(hour);
        this.time.setMinutes(minute);
        this.setDateTime(arkeia.widget.ArkeiaTimePicker.util.toRfcDateTime(this.time));

		this.onValueChanged(this.time);

	},
	
	onValueChanged: function(/*Date*/date) {
		//summary: the set date event handler
	}
});

arkeia.widget.ArkeiaTimePicker.util = new function() {

	this.toRfcDateTime = function(jsDate) {

		if(!jsDate) {
			jsDate = new Date();
		}

		jsDate.setSeconds(0);

		return dojo.date.toRfc3339(jsDate);
	}

	this.fromRfcDateTime = function(rfcDate) {

        var tempDate = new Date();

        if(rfcDate && rfcDate.indexOf("T")!=-1) {

            var tempTime = rfcDate.split("T")[1].split(":");

            var tempDate = new Date();
            tempDate.setHours(tempTime[0]);
            tempDate.setMinutes(tempTime[1]);

        }

		return tempDate;
	}

}
