<?php

/**
 * Arkeia 
 *
 * @category   Arkeia
 * @package    Arkeia_Auth
 * @copyright  Copyright (c) 2007 Arkeia Software (http://www.arkeia.com)
 * @version    $Id: Arkws.php,v 1.12 2008/04/11 18:21:28 qlabadens Exp $
 */


/**
 * @see Zend_Auth_Adapter_Interface
 */
require_once 'Zend/Auth/Adapter/Interface.php';


/**
 * @category   Arkeia
 * @package    Arkeia_Auth
 * @copyright  Copyright (c) 2007 Arkeia Software (http://www.arkeia.com)
 */
class Arkeia_Auth_Adapter_Arkws implements Zend_Auth_Adapter_Interface
{

    /**
     * Arkeia authentication user
     *
     * @var string
     */
    protected $_username;

    /**
     * Password for the user
     *
     * @var string
     */
    protected $_password;

    /**
     * Adminstration mode 
     *
     * @var int
     */
    protected $_admin;

    /**
     * Arkeia default role
     *
     * @var string
     */
    protected $_defaultrole = "CONNECTED";

    /**
     * Sets adapter options
     *
     * @param  mixed $username
     * @param  mixed $password
     * @param  mixed $admin
     * @return void
     */
    public function __construct($username = null, $password = null, $admin=0, $sessnum=1)
    {
        $options = array('username', 'password', 'admin', 'sessnum');
        foreach ($options as $option) {
            if (null !== $$option) {
                $methodName = 'set' . ucfirst($option);
                $this->$methodName($$option);
            }
        }
    }

    /**
     * Returns the username option value or null if it has not yet been set
     *
     * @return string|null
     */
    public function getUsername()
    {
        return $this->_username;
    }

    /**
     * Sets the username option value
     *
     * @param  mixed $username
     * @return Arkeia_Auth_Adapter_Arkws Provides a fluent interface
     */
    public function setUsername($username)
    {
        $this->_username = (string) $username;
        return $this;
    }

    /**
     * Returns the password option value or null if it has not yet been set
     *
     * @return string|null
     */
    public function getPassword()
    {
        return $this->_password;
    }

    /**
     * Sets the password option value
     *
     * @param  mixed $password
     * @return Arkeia_Auth_Adapter_Arkws Provides a fluent interface
     */
    public function setPassword($password)
    {
        $this->_password = (string) $password;
        return $this;
    }

    /**
     * Returns the admin option value or null if it has not yet been set
     *
     * @return int|null
     */
    public function getAdmin()
    {
        return $this->_admin;
    }

    /**
     * Sets the admin option value
     *
     * @param  mixed $admin
     * @return Arkeia_Auth_Adapter_Arkws Provides a fluent interface
     */
    public function setAdmin($admin)
    {
        $this->_admin = (string) $admin;
        return $this;
    }

    public function getSessnum()
    {
        return $this->_sessum;
    }
    public function setSessnum($sessnum)
    {
        $this->_sessnum = $sessnum;
        return $this;
    }

    /**
     * Defined by Zend_Auth_Adapter_Interface
     *
     * @throws Zend_Auth_Adapter_Exception
     * @return Zend_Auth_Result
     */
    public function authenticate()
    {
        $optionsRequired = array('username', 'password');
        foreach ($optionsRequired as $optionRequired) {
            if (null === $this->{"_$optionRequired"}) {
                /**
                 * @see Zend_Auth_Adapter_Exception
                 */
                require_once 'Zend/Auth/Adapter/Exception.php';
                throw new Zend_Auth_Adapter_Exception("Option '$optionRequired' must be set before authentication");
            }
        }

        $result = array(
            'isValid'  => false,
            'identity' => array(
                'role'    => $this->_defaultrole,
                'username' => $this->_username,
                'password' => $this->_password,
                'admin' => $this->_admin,
                'sessnum' => $this->_sessnum,
                'appliances' => null,
                ),
            'messages' => array()
            );
        
       require_once 'Arkeia/Soap.php';

       $acx = Arkeia_Soap::getInstance();
       $connected = false;
       $login_result = $acx->login($this->_username, $this->_password, $this->_admin, $this->_sessnum);
       if (!is_soap_fault($login_result)) {
           if (isset($login_result['param0']) && isset($login_result['param1']) &&
              ($login_result['param0'] == 12) && $login_result['param1'] == 0) {
               $connected = true;
               $trans_version = 1;
               $vector = array();
               if (!$this->_admin) { $trans = "USR_VALIDATE"; }
               else { $trans = "LDAP_LOGIN"; }
               $login_result = $acx->new_trans(
                    $trans, $trans_version, $this->_sessnum, $vector);
           }
       }
       if (is_soap_fault($login_result)) {
           if ($login_result->detail) {
               $result['messages'][] = $login_result->detail;
           }
           else {
               $result['messages'][0] = "{$login_result->faultcode}";
               $result['messages'][1] = "{$login_result->faultstring}";
           }
       } else {
           if (isset($login_result->param0)){
               if ($login_result->param0 == 0) {
                   if (isset($login_result->param2)) {
                       $list = $login_result->param2;
                       if ($this->_admin) $result['identity']['appliances'] = $list;
		       if (isset($list->NOT_ENOUGH_LICENSE)) {
			   $result['identity']['NOT_ENOUGH_LICENSE'] = true;
		       } else if (is_array($list->item)) {
                           foreach($list->item as $item) {
                               if (isset($item->ROLE)) {
                                   $result['identity']['role'] = $item->ROLE;
                                   break;
                               }
                           }
                       } else {
                           if (isset($list->item->ROLE)) {
                               $result['identity']['role'] = $list->item->ROLE;
                           } else {
			       if ($list == '' && $result['identity']['admin']) {
				   $result['identity']['NO_APPLIANCE_CONFIGURED'] = true;
			       }
			   }
			   
                       }
                   } else if ($this->_admin){
                       error_log("Arkws admin mode and no param2!");
                       $result['identity']['appliances'] = (object) NULL;
                   }
                   $result['isValid'] = true;
                   $result['messages'][0] = "param0: {$login_result->param0}";
                   $result['messages'][1] = "param1: {$login_result->param1}";
		   if (isset($list->NOT_ENOUGH_LICENSE)) {
		     $result['messages'][2] = "NOT_ENOUGH_LICENSE";
		   } else if (isset($result['identity']['NO_APPLIANCE_CONFIGURED']) && $result['identity']['NO_APPLIANCE_CONFIGURED']) {
		     $result['messages'][2] = "NO_APPLIANCE_CONFIGURED";
		   }
               } else {
                   $result['messages'][0] = "Error code {$login_result->param0}";
                   $result['messages'][1] = "param1: {$login_result->param1}";
               }
           } else if ($login_result->param1) {
               $result['messages'][] = $login_result->param1;
           } else {
               $result['messages'][0] = "No Answer";
           }
       }
       if ($connected && !$result['isValid']) $acx->logout($this->_sessnum);
       return new Zend_Auth_Result($result['isValid'], $result['identity'], $result['messages']);

    }
}
